// Copyright 2016-2017 Luke Shumaker
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package sd_daemon_test

import (
	"log/syslog"
	"testing"

	"git.lukeshu.com/go/libsystemd/sd_daemon"
)

type cbWriter func(p []byte) (n int, err error)

func (cb cbWriter) Write(p []byte) (n int, err error) {
	return cb(p)
}

func TestLog(t *testing.T) {
	var written []byte
	log := sd_daemon.NewLogger(cbWriter(func(p []byte) (n int, err error) {
		written = p
		return len(p), nil
	}))

	type testcase struct {
		pri syslog.Priority
		msg string
	}

	testcases := map[testcase]string{
		{6, "foo"}:             "<6>foo\n",
		{6, "foo\n"}:           "<6>foo\n",
		{6, "foo\nbar"}:        "<6>foo\n<6>bar\n",
		{6, "foo\nbar\n"}:      "<6>foo\n<6>bar\n",
		{6, "foo\nbar\nbaz"}:   "<6>foo\n<6>bar\n<6>baz\n",
		{6, "foo\nbar\nbaz\n"}: "<6>foo\n<6>bar\n<6>baz\n",

		{0, "foo"}:  "<0>foo\n",
		{1, "foo"}:  "<1>foo\n",
		{10, "foo"}: "<10>foo\n",
	}

	for in, out := range testcases {
		written = nil
		n, err := log.LogString(in.pri, in.msg)
		if n != len(out) || string(written) != out || err != nil {
			t.Errorf("LogString(%#v, %#v)\n -> expected:{%#v, %#v, %#v}\n -> got:{%#v, %#v, %#v}\n",
				in.pri, in.msg,
				len(out), nil, out,
				n, err, string(written))
		}
		written = nil
		n, err = log.LogBytes(in.pri, []byte(in.msg))
		if n != len(out) || string(written) != out || err != nil {
			t.Errorf("LogBytes(%#v, %#v)\n -> expected:{%#v, %#v, %#v}\n -> got:{%#v, %#v, %#v}\n",
				in.pri, in.msg,
				len(out), nil, out,
				n, err, string(written))
		}
	}
}
